from typing import Any, Callable

from ..asgi.router import ControllerAnnotation, Route
from ..exceptions import MethodNotFoundError
from ..method import MethodType


class _RouteDecorator:
    def __getattr__(self, attr_name: str) -> Any:
        if attr_name == '__name__':
            return type(self).__name__

        method = attr_name.upper()

        if method not in MethodType.__members__:
            raise MethodNotFoundError(attr_name)

        def decorator(
            path_pattern: str,
            query: bool = False,
            headers: bool = False,
            body: bool = False,
        ) -> Callable[[ControllerAnnotation], ControllerAnnotation]:
            if '{' in path_pattern and '}' in path_pattern:
                has_path_args = True
            else:
                has_path_args = False

            def wrapper(
                controller: ControllerAnnotation,
            ) -> ControllerAnnotation:
                route = Route(
                    path_pattern=path_pattern,
                    method=MethodType[method],
                    controller=controller,  # type: ignore
                    has_path_args=has_path_args,
                    has_query=query,
                    has_headers=headers,
                    has_body=body,
                )

                if hasattr(controller, 'routes'):
                    controller.routes.append(route)  # type: ignore
                else:
                    controller.routes = [route]  # type: ignore

                return controller

            return wrapper

        return decorator


route = _RouteDecorator()



#!/usr/bin/env python3.7

import argparse
import re


def main(args):
    with open(args.source) as file:
        content = file.read()

    for pattern, filename in re.findall(r'(\{!(.*)!\})', content):
        with open(filename) as file:
            match_content = file.read()

        dest_content = re.sub(pattern, match_content, content)

        with open(args.dest, mode='w') as file:
            file.write(dest_content)

        with open(args.dest) as file:
            content = file.read() 


if __name__ == '__main__':
    parser = argparse.ArgumentParser(
        description="Replace placeholders with it's file contents"
    )
    parser.add_argument(
        'source',
        metavar='SOURCE',
        type=str,
        help='File to parse'
    )
    parser.add_argument(
        'dest',
        metavar='DEST',
        type=str,
        help='destination file'
    )
    args = parser.parse_args()
    main(args)



import os
import time

from apidaora import appdaora, route


REDIS_URI = f"redis://{os.environ.get('REDIS', '')}"


@route.background('/hello', tasks_repository=REDIS_URI)
def hello_task(name: str) -> str:
    time.sleep(1)
    return f'Hello {name}!'


app = appdaora(hello_task)



import io
from typing import IO, Optional

from dictdaora import DictDaora


try:
    import gzip
except Exception:
    gzip = None  # type: ignore


class GZipFactory(DictDaora):
    mode: str = 'rb'
    compresslevel: int = 9
    encoding: Optional[str] = None
    errors: Optional[str] = None
    newline: Optional[str] = None
    value: bytes

    def open(self) -> IO[str]:
        if self.value:
            return gzip.open(
                io.BytesIO(self.value),
                self.mode,
                self.compresslevel,
                self.encoding,
                self.errors,
                self.newline,
            )

        raise ValueError(self.value)



"""
ASGI App using dataclasses module for request/response objects
"""

__version__ = '0.22.1'

from apidaora.app import appdaora
from apidaora.bodies import GZipFactory
from apidaora.class_controller import ClassController
from apidaora.content import ContentType
from apidaora.exceptions import BadRequestError
from apidaora.header import Header
from apidaora.method import MethodType
from apidaora.middlewares import (
    CorsMiddleware,
    LockRequestMiddleware,
    MiddlewareRequest,
    Middlewares,
)
from apidaora.responses import (
    Response,
    html,
    json,
    no_content,
    not_found,
    text,
)
from apidaora.route.decorator import route


__all__ = [
    'BadRequestError',
    'GZipFactory',
    'ContentType',
    'MethodType',
    'appdaora',
    'Header',
    'html',
    'json',
    'text',
    'route',
    'no_content',
    'not_found',
    'Response',
    'Middlewares',
    'CorsMiddleware',
    'MiddlewareRequest',
    'ClassController',
    'LockRequestMiddleware',
]



from typing import List

from apidaora import LockRequestMiddleware, Middlewares, appdaora, route


prevent_request_middleware = LockRequestMiddleware()


@route.post(
    '/lock-request',
    middlewares=Middlewares(
        post_routing=[prevent_request_middleware.lock],
        pre_execution=[prevent_request_middleware.unlock_pre_execution],
    ),
)
async def lock_controller(body: List[int]) -> int:
    return len(body)


@route.post(
    '/lock-request-post',
    middlewares=Middlewares(
        post_routing=[prevent_request_middleware.lock],
        post_execution=[prevent_request_middleware.unlock_post_execution],
    ),
)
async def lock_post_controller(body: List[int]) -> int:
    return len(body)


app = appdaora([lock_controller, lock_post_controller])



from apidaora import ClassController, appdaora


class HelloWorld(ClassController, path='/hello'):
    def get(self) -> str:
        return 'Hello World!'

    def post(self, body: str) -> str:
        return f'Hello {body}!'


class HelloDuckType:
    path = '/hello-duck'

    def get(self) -> str:
        return 'Hello Duck!'


app = appdaora([HelloWorld, HelloDuckType])
